@@ -24,6 +24,16 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: asner.h,v $
+ * Revision 1.36  2003/02/26 01:57:44  robertj
+ * Added XML encoding rules to ASN system, thanks Federico Pinna
+ *
+ * Revision 1.35  2003/02/01 13:25:52  robertj
+ * Added function to add new elements directly to ASN array.
+ *
+ * Revision 1.34  2003/01/24 23:43:43  robertj
+ * Fixed subtle problems with the use of MAX keyword for unsigned numbers,
+ *   should beUINT_MAX not INT_MAX, thanks Stevie Gray for pointing it out.
+ *
  * Revision 1.33  2002/11/26 23:29:18  robertj
  * Added missing const to DecodeSubType() function.
  *
@@ -144,6 +154,10 @@
 class PASN_Stream;
 class PBER_Stream;
 class PPER_Stream;
+#if P_EXPAT
+class PXER_Stream;
+class PXMLElement;
+#endif
 
 
 /////////////////////////////////////////////////////////////////////////////
@@ -220,8 +234,8 @@ class PASN_Object : public PObject
     enum MaximumValueTag { MaximumValue = INT_MAX };
     void SetConstraints(ConstraintType type, int value)
       { SetConstraintBounds(type, value, value); }
-    void SetConstraints(ConstraintType, int lower, MaximumValueTag upper)
-      { SetConstraintBounds(PartiallyConstrained, (int)lower, (unsigned)upper); }
+    void SetConstraints(ConstraintType, int lower, MaximumValueTag /*upper*/)
+      { SetConstraintBounds(PartiallyConstrained, (int)lower, lower < 0 ? INT_MAX : UINT_MAX); }
     void SetConstraints(ConstraintType, MinimumValueTag lower, unsigned upper)
       { SetConstraintBounds(PartiallyConstrained, (int)lower, (unsigned)upper); }
     void SetConstraints(ConstraintType, MinimumValueTag lower, MaximumValueTag upper)
@@ -394,6 +408,10 @@ class PASN_Enumeration : public PASN_Object
 
     BOOL DecodePER(PPER_Stream & strm);
     void EncodePER(PPER_Stream & strm) const;
+#if P_EXPAT
+    virtual BOOL DecodeXER(PXER_Stream & strm);
+    virtual void EncodeXER(PXER_Stream & strm) const;
+#endif
 
   protected:
     unsigned maxEnumValue;
@@ -811,6 +829,10 @@ class PASN_Choice : public PASN_Object
 
     virtual BOOL DecodePER(PPER_Stream &);
     virtual void EncodePER(PPER_Stream &) const;
+#if P_EXPAT
+    BOOL DecodeXER(PXER_Stream &);
+    void EncodeXER(PXER_Stream &) const;
+#endif
 
 #ifdef P_MACOSX
   #warning making PASN_Choice public
@@ -888,6 +910,15 @@ class PASN_Sequence : public PASN_Object
     BOOL UnknownExtensionsDecodePER(PPER_Stream & strm);
     void UnknownExtensionsEncodePER(PPER_Stream & strm) const;
 
+#if P_EXPAT
+    virtual BOOL PreambleDecodeXER(PXER_Stream & strm);
+    virtual void PreambleEncodeXER(PXER_Stream & strm) const;
+    virtual BOOL KnownExtensionDecodeXER(PXER_Stream & strm, PINDEX fld, PASN_Object & field);
+    virtual void KnownExtensionEncodeXER(PXER_Stream & strm, PINDEX fld, const PASN_Object & field) const;
+    virtual BOOL UnknownExtensionsDecodeXER(PXER_Stream & strm);
+    virtual void UnknownExtensionsEncodeXER(PXER_Stream & strm) const;
+#endif
+
   protected:
     BOOL NoExtensionsToDecode(PPER_Stream & strm);
     BOOL NoExtensionsToEncode(PPER_Stream & strm);
@@ -925,6 +956,7 @@ class PASN_Array : public PASN_ConstrainedObject
     PINDEX GetSize() const { return array.GetSize(); }
     BOOL SetSize(PINDEX newSize);
     PASN_Object & operator[](PINDEX i) const { return array[i]; }
+    void Append(PASN_Object * obj) { array.SetAt(array.GetSize(), obj); }
     void RemoveAt(PINDEX i) { array.RemoveAt(i); }
     void RemoveAll() { array.RemoveAll(); }
 
@@ -1159,4 +1191,58 @@ class PPER_Stream : public PASN_Stream
 };
 
 
+#if P_EXPAT
+/** Class for ASN XML Encoding Rules stream.
+*/
+class PXER_Stream : public PASN_Stream
+{
+    PCLASSINFO(PXER_Stream, PASN_Stream);
+  public:
+    PXER_Stream(PXMLElement * elem);
+    PXER_Stream(PXMLElement * elem, const PBYTEArray & bytes);
+    PXER_Stream(PXMLElement * elem, const BYTE * buf, PINDEX size);
+
+    virtual BOOL Read(PChannel & chan);
+    virtual BOOL Write(PChannel & chan);
+
+    virtual BOOL NullDecode(PASN_Null &);
+    virtual void NullEncode(const PASN_Null &);
+    virtual BOOL BooleanDecode(PASN_Boolean &);
+    virtual void BooleanEncode(const PASN_Boolean &);
+    virtual BOOL IntegerDecode(PASN_Integer &);
+    virtual void IntegerEncode(const PASN_Integer &);
+    virtual BOOL EnumerationDecode(PASN_Enumeration &);
+    virtual void EnumerationEncode(const PASN_Enumeration &);
+    virtual BOOL RealDecode(PASN_Real &);
+    virtual void RealEncode(const PASN_Real &);
+    virtual BOOL ObjectIdDecode(PASN_ObjectId &);
+    virtual void ObjectIdEncode(const PASN_ObjectId &);
+    virtual BOOL BitStringDecode(PASN_BitString &);
+    virtual void BitStringEncode(const PASN_BitString &);
+    virtual BOOL OctetStringDecode(PASN_OctetString &);
+    virtual void OctetStringEncode(const PASN_OctetString &);
+    virtual BOOL ConstrainedStringDecode(PASN_ConstrainedString &);
+    virtual void ConstrainedStringEncode(const PASN_ConstrainedString &);
+    virtual BOOL BMPStringDecode(PASN_BMPString &);
+    virtual void BMPStringEncode(const PASN_BMPString &);
+    virtual BOOL ChoiceDecode(PASN_Choice &);
+    virtual void ChoiceEncode(const PASN_Choice &);
+    virtual BOOL ArrayDecode(PASN_Array &);
+    virtual void ArrayEncode(const PASN_Array &);
+    virtual BOOL SequencePreambleDecode(PASN_Sequence &);
+    virtual void SequencePreambleEncode(const PASN_Sequence &);
+    virtual BOOL SequenceKnownDecode(PASN_Sequence &, PINDEX, PASN_Object &);
+    virtual void SequenceKnownEncode(const PASN_Sequence &, PINDEX, const PASN_Object &);
+    virtual BOOL SequenceUnknownDecode(PASN_Sequence &);
+    virtual void SequenceUnknownEncode(const PASN_Sequence &);
+
+    PXMLElement * GetCurrentElement()                   { return position; }
+    PXMLElement * SetCurrentElement(PXMLElement * elem) { return position = elem; }
+
+  protected:
+    PXMLElement * position;
+};
+#endif // P_EXPAT
+
+
 #endif // _ASNER_H
