@@ -5,7 +5,7 @@
  *
  * Portable Windows Library
  *
- * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
+ * Copyright (c) 1993-2002 Equivalence Pty. Ltd.
  *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.0 (the "License"); you may not use this file except in
@@ -21,12 +21,35 @@
  *
  * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
  *
- * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
- * All Rights Reserved.
- *
  * Contributor(s): ______________________________________.
  *
  * $Log: http.cxx,v $
+ * Revision 1.79  2002/12/02 00:17:03  robertj
+ * Fixed URL parsing/display problems with non-path URL type eg mailto
+ *
+ * Revision 1.78  2002/11/22 06:16:49  robertj
+ * Fixed usage of URI (relative http/https URL).
+ *
+ * Revision 1.77  2002/11/20 02:10:56  robertj
+ * Fixed some more realtive/absolute path issues.
+ *
+ * Revision 1.76  2002/11/20 01:01:49  robertj
+ * Fixed GNU compatibility
+ *
+ * Revision 1.75  2002/11/20 00:49:37  robertj
+ * Fixed correct interpretation of url re double slashes as per latest RFC,
+ *   including file: mapping and relative paths. Probably still more to do.
+ *
+ * Revision 1.74  2002/11/19 22:45:03  robertj
+ * Fixed support for file: scheme under unix
+ *
+ * Revision 1.73  2002/11/19 10:36:50  robertj
+ * Added functions to set anf get "file:" URL. as PFilePath and do the right
+ *   things with platform dependent directory components.
+ *
+ * Revision 1.72  2002/11/06 22:47:25  robertj
+ * Fixed header comment (copyright etc)
+ *
  * Revision 1.71  2002/09/23 07:17:24  robertj
  * Changes to allow winsock2 to be included.
  *
@@ -306,60 +329,61 @@
 #define	DEFAULT_H323_PORT       1720
 #define	DEFAULT_SIP_PORT        5060
 
-enum SchemeFormat {
-  HostPort = 1,
-  UserPasswordHostPort = 2,
-  HostOnly = 3,
-  Other = 4,
-  NoHost = 5
+
+struct schemeStruct {
+  const char * name;
+  BOOL hasUserPassword;
+  BOOL hasHostPort;
+  BOOL hasQuery;
+  BOOL hasParameters;
+  BOOL hasFragments;
+  BOOL hasPath;
+  BOOL relativeImpliesScheme;
+  WORD defaultPort;
 };
 
-class schemeStruct {
-  public:
-    const char * name;
-    int  type;
-    BOOL hasDoubleSlash;
-    WORD defaultPort;
-};
+#define DEFAULT_SCHEME 0
+#define FILE_SCHEME    1
 
 static schemeStruct const schemeInfo[] = {
-  { "http",      UserPasswordHostPort, TRUE, DEFAULT_HTTP_PORT },
-  { "https",     HostPort, TRUE, DEFAULT_HTTPS_PORT },
-  { "gopher",    HostPort, TRUE, DEFAULT_GOPHER_PORT },
-  { "wais",      HostPort, TRUE, DEFAULT_WAIS_PORT },
-  { "nntp",      HostPort, TRUE, DEFAULT_NNTP_PORT },
-  { "prospero",  HostPort, TRUE, DEFAULT_PROSPERO_PORT },
-  { "rtsp",      HostPort, TRUE, DEFAULT_RTSP_PORT },
-  { "rtspu",     HostPort, TRUE, DEFAULT_RTSPU_PORT },
+//  scheme       user   host   query  params frags  path   rel    port
+  { "http",      TRUE,  TRUE,  TRUE,  TRUE,  TRUE,  TRUE,  TRUE,  DEFAULT_HTTP_PORT     }, // Must be first
+  { "file",      FALSE, TRUE,  FALSE, FALSE, FALSE, TRUE,  FALSE, 0                     }, // Must be second
+  { "https",     FALSE, TRUE,  TRUE,  TRUE,  TRUE,  TRUE,  TRUE,  DEFAULT_HTTPS_PORT    },
+  { "gopher",    FALSE, TRUE,  FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_GOPHER_PORT   },
+  { "wais",      FALSE, TRUE,  FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_WAIS_PORT     },
+  { "nntp",      FALSE, TRUE,  FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_NNTP_PORT     },
+  { "prospero",  FALSE, TRUE,  FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_PROSPERO_PORT },
+  { "rtsp",      FALSE, TRUE,  FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_RTSP_PORT     },
+  { "rtspu",     FALSE, TRUE,  FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_RTSPU_PORT    },
 
-  { "ftp",       UserPasswordHostPort, TRUE, DEFAULT_FTP_PORT },
-  { "telnet",    UserPasswordHostPort, TRUE, DEFAULT_TELNET_PORT },
-  { "file",      NoHost,               TRUE },
-  { "mailto",    Other, FALSE},
-  { "news",      Other, FALSE},
-  { "h323",      UserPasswordHostPort, FALSE, DEFAULT_H323_PORT },
-  { "sip",       UserPasswordHostPort, FALSE, DEFAULT_SIP_PORT },
-  { NULL }
+  { "ftp",       TRUE,  TRUE,  FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_FTP_PORT      },
+  { "telnet",    TRUE,  TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE, DEFAULT_TELNET_PORT   },
+  { "mailto",    FALSE, FALSE, TRUE,  FALSE, FALSE, FALSE, FALSE, 0                     },
+  { "news",      FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, 0                     },
+  { "h323",      TRUE,  TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, DEFAULT_H323_PORT     },
+  { "sip",       TRUE,  TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, DEFAULT_SIP_PORT      },
+  { NULL,        FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, 0                     }
 };
 
-static schemeStruct const defaultSchemeInfo = { "other", Other, FALSE};
-
-static const schemeStruct & GetSchemeInfo(const PString & scheme)
+static const schemeStruct & GetSchemeInfo(const PCaselessString & scheme)
 {
   PINDEX i;
-  for (i = 0; schemeInfo[i].name != NULL; i++)
-    if (scheme *= schemeInfo[i].name)
+  for (i = 0; schemeInfo[i].name != NULL; i++) {
+    if (scheme == schemeInfo[i].name)
       return schemeInfo[i];
-  return defaultSchemeInfo;
+  }
+  return schemeInfo[i];
 }
 
 //////////////////////////////////////////////////////////////////////////////
 // PURL
 
 PURL::PURL()
+  : scheme(schemeInfo[DEFAULT_SCHEME].name),
+    port(0),
+    relativePath(FALSE)
 {
-  scheme = schemeInfo[0].name;
-  port   = 0;
 }
 
 
@@ -375,6 +399,23 @@ PURL::PURL(const PString & str)
 }
 
 
+PURL::PURL(const PFilePath & filePath)
+  : scheme(schemeInfo[FILE_SCHEME].name),
+    port(0),
+    relativePath(FALSE)
+{
+  PStringArray pathArray = filePath.GetDirectory().GetPath();
+  hostname = pathArray[0];
+
+  PINDEX i;
+  for (i = 1; i < pathArray.GetSize(); i++)
+    pathArray[i-1] = pathArray[i];
+  pathArray[i-1] = filePath.GetFileName();
+
+  SetPath(pathArray);
+}
+
+
 PObject::Comparison PURL::Compare(const PObject & obj) const
 {
   PAssert(obj.IsDescendant(PURL::Class()), PInvalidCast);
@@ -503,6 +544,7 @@ void PURL::Parse(const char * cstr)
   path.SetSize(0);
   queryVars.RemoveAll();
   port = 0;
+  relativePath = FALSE;
 
   // copy the string so we can take bits off it
   while (isspace(*cstr))
@@ -514,7 +556,7 @@ void PURL::Parse(const char * cstr)
   static const PString reservedChars = "=;/#?";
 
   // determine if the URL has a scheme
-  scheme = "";
+  scheme = PString::Empty();
   if (isalpha(url[0])) {
     for (pos = 0; url[pos] != '\0' &&
                           reservedChars.Find(url[pos]) == P_MAX_INDEX; pos++) {
@@ -528,101 +570,127 @@ void PURL::Parse(const char * cstr)
 
   // if there is no scheme, then default to http for the local
   // on the default port
-  if (scheme.IsEmpty()) {
-    scheme = schemeInfo[0].name;
-    if (url.GetLength() > 2 && url[0] == '/' && url[1] == '/') 
-      url.Delete(0, 2);
-  } else {
+  if (scheme.IsEmpty())
+    scheme = schemeInfo[DEFAULT_SCHEME].name;
 
-    // get information which tells us how to parse URL for this
-    // particular scheme
-    const schemeStruct & schemeInfo = GetSchemeInfo(scheme);
+  // get information which tells us how to parse URL for this
+  // particular scheme
+  const schemeStruct & schemeInfo = GetSchemeInfo(scheme);
     
-    // if the URL should have leading slash, then remove it if it has one
-    if (schemeInfo.hasDoubleSlash &&
-      url.GetLength() > 2 && url[0] == '/' && url[1] == '/') 
-    url.Delete(0, 2);
+  // if the URL should have leading slash, then remove it if it has one
+  if (schemeInfo.hasHostPort && schemeInfo.hasPath) {
+    if (url.GetLength() > 2 && url[0] == '/' && url[1] == '/')
+      url.Delete(0, 2);
+    else
+      relativePath = TRUE;
+  }
 
-    // if the rest of the URL isn't a path, then we are finished!
-    if (schemeInfo.type == Other) {
-      pathStr = url;
-      return;
+  // parse user/password/host/port
+  if (!relativePath && schemeInfo.hasHostPort) {
+    pos = url.FindOneOf("/;?#");
+    PString uphp = url.Left(pos);
+    if (pos != P_MAX_INDEX)
+      url.Delete(0, pos);
+    else
+      url = PString();
+
+    // if the URL is of type HostOnly, then this is the hostname
+    if (schemeInfo.defaultPort == 0)
+      hostname = UntranslateString(uphp, LoginTranslation);
+
+    // if the URL is of type UserPasswordHostPort, then parse it
+    if (schemeInfo.hasUserPassword) {
+      // extract username and password
+      PINDEX pos2 = uphp.Find('@');
+      if (pos2 != P_MAX_INDEX && pos2 > 0) {
+        PINDEX pos3 = uphp.Find(':');
+        // if no password...
+        if (pos3 > pos2)
+          username = UntranslateString(uphp(0, pos2-1), LoginTranslation);
+        else {
+          username = UntranslateString(uphp(0, pos3-1), LoginTranslation);
+          password = UntranslateString(uphp(pos3+1, pos2-1), LoginTranslation);
+        }
+        uphp.Delete(0, pos2+1);
+      }
     }
 
-    // parse user/password/host/port
-    if (schemeInfo.type == HostPort ||
-        schemeInfo.type == UserPasswordHostPort ||
-        schemeInfo.type == HostOnly) {
-      pos = url.FindOneOf("/;?#");
-      PString uphp = url.Left(pos);
-      if (pos != P_MAX_INDEX)
-        url.Delete(0, pos);
-      else
-        url = PString();
-
-      // if the URL is of type HostOnly, then this is the hostname
-      if (schemeInfo.type == HostOnly) {
+    // determine if the URL has a port number
+    if (schemeInfo.defaultPort != 0) {
+      pos = uphp.Find(':');
+      if (pos == P_MAX_INDEX) {
         hostname = UntranslateString(uphp, LoginTranslation);
-      } 
-
-      // if the URL is of type UserPasswordHostPort, then parse it
-      if (schemeInfo.type == UserPasswordHostPort) {
-
-        // extract username and password
-        PINDEX pos2 = uphp.Find('@');
-        if (pos2 != P_MAX_INDEX && pos2 > 0) {
-          PINDEX pos3 = uphp.Find(":");
-          // if no password...
-          if (pos3 > pos2)
-            username = UntranslateString(uphp(0, pos2-1), LoginTranslation);
-          else {
-            username = UntranslateString(uphp(0, pos3-1), LoginTranslation);
-            password = UntranslateString(uphp(pos3+1, pos2-1), LoginTranslation);
-          }
-          uphp.Delete(0, pos2+1);
-        }
+        port = schemeInfo.defaultPort;
+      }
+      else {
+        hostname = UntranslateString(uphp.Left(pos), LoginTranslation);
+        port = (WORD)uphp(pos+1, P_MAX_INDEX).AsInteger();
       }
 
-      // determine if the URL has a port number
-      if (schemeInfo.type == HostPort ||
-          schemeInfo.type == UserPasswordHostPort) {
-        pos = uphp.Find(":");
-        if (pos == P_MAX_INDEX) {
-          hostname = UntranslateString(uphp, LoginTranslation);
-          port = schemeInfo.defaultPort;
-        } else {
-          hostname = UntranslateString(uphp.Left(pos), LoginTranslation);
-          port = (WORD)uphp(pos+1, P_MAX_INDEX).AsInteger();
-        }
-        if (hostname.IsEmpty())
-          hostname = PIPSocket::GetHostName();
-      }
+      if (hostname.IsEmpty())
+        hostname = PIPSocket::GetHostName();
     }
   }
 
-  // chop off any trailing query
-  pos = url.Find('?');
-  if (pos != P_MAX_INDEX /* && pos > 0 */) {
-    SplitQueryVars(url(pos+1, P_MAX_INDEX), queryVars);
-    url.Delete(pos, P_MAX_INDEX);
+  if (schemeInfo.hasQuery) {
+    // chop off any trailing query
+    pos = url.Find('?');
+    if (pos != P_MAX_INDEX /* && pos > 0 */) {
+      SplitQueryVars(url(pos+1, P_MAX_INDEX), queryVars);
+      url.Delete(pos, P_MAX_INDEX);
+    }
   }
 
-  // chop off any trailing parameters
-  pos = url.Find(';');
-  if (pos != P_MAX_INDEX /* && pos > 0 */) {
-    SplitVars(url(pos+1, P_MAX_INDEX), paramVars, ';', '=');
-    url.Delete(pos, P_MAX_INDEX);
+  if (schemeInfo.hasParameters) {
+    // chop off any trailing parameters
+    pos = url.Find(';');
+    if (pos != P_MAX_INDEX /* && pos > 0 */) {
+      SplitVars(url(pos+1, P_MAX_INDEX), paramVars, ';', '=');
+      url.Delete(pos, P_MAX_INDEX);
+    }
   }
 
-  // chop off any trailing fragment
-  pos = url.Find('#');
-  if (pos != P_MAX_INDEX /* && pos > 0 */) {
-    fragment = UntranslateString(url(pos+1, P_MAX_INDEX), PathTranslation);
-    url.Delete(pos, P_MAX_INDEX);
+  if (schemeInfo.hasFragments) {
+    // chop off any trailing fragment
+    pos = url.Find('#');
+    if (pos != P_MAX_INDEX /* && pos > 0 */) {
+      fragment = UntranslateString(url(pos+1, P_MAX_INDEX), PathTranslation);
+      url.Delete(pos, P_MAX_INDEX);
+    }
   }
 
-  // the hierarchy is what is left
-  SetPathStr(url);
+  if (schemeInfo.hasPath)
+    SetPathStr(url);   // the hierarchy is what is left
+  else {
+  // if the rest of the URL isn't a path, then we are finished!
+    pathStr = UntranslateString(url, PathTranslation);
+    Recalculate();
+  }
+}
+
+
+PFilePath PURL::AsFilePath() const
+{
+  if (scheme != schemeInfo[FILE_SCHEME].name)
+    return PString::Empty();
+
+  PStringStream str;
+
+  if (relativePath) {
+    for (PINDEX i = 0; i < path.GetSize(); i++) {
+      if (i > 0)
+        str << PDIR_SEPARATOR;
+      str << path[i];
+    }
+  }
+  else {
+    if (hostname != "localhost")
+      str << hostname;
+    for (PINDEX i = 0; i < path.GetSize(); i++)
+      str << PDIR_SEPARATOR << path[i];
+  }
+
+  return str;
 }
 
 
@@ -634,54 +702,53 @@ PString PURL::AsString(UrlFormat fmt) const
   if (fmt == FullURL)
     return urlString;
 
+  if (scheme.IsEmpty())
+    return PString::Empty();
+
+  const schemeStruct & schemeInfo = GetSchemeInfo(scheme);
+
   if (fmt == HostPortOnly) {
-    if (scheme.IsEmpty())
-      return PString::Empty();
-
     str << scheme << ':';
-    const schemeStruct & schemeInfo = GetSchemeInfo(scheme);
 
-    if (schemeInfo.hasDoubleSlash)
+    if (relativePath) {
+      if (schemeInfo.relativeImpliesScheme)
+        return PString::Empty();
+      return str;
+    }
+
+    if (schemeInfo.hasPath && schemeInfo.hasHostPort)
       str << "//";
 
-    if (schemeInfo.type == Other) 
-      str << pathStr;
-    else {
-      if (schemeInfo.type == HostOnly)
-        str << hostname;
-
-      if (schemeInfo.type == UserPasswordHostPort) {
-        if (!username) {
-          str << TranslateString(username, LoginTranslation);
-          if (!password)
-            str << ':' << TranslateString(password, LoginTranslation);
-          str << '@';
-        }
-      }
-
-      if (schemeInfo.type == HostPort || schemeInfo.type == UserPasswordHostPort) {
-        if (hostname.IsEmpty())
-          str = PString();
-        else {
-          str << hostname;
-          if (port != schemeInfo.defaultPort)
-            str << ':' << port;
-        }
+    if (schemeInfo.hasUserPassword) {
+      if (!username) {
+        str << TranslateString(username, LoginTranslation);
+        if (!password)
+          str << ':' << TranslateString(password, LoginTranslation);
+        str << '@';
       }
     }
+
+    if (schemeInfo.hasHostPort)
+      str << hostname;
+
+    if (schemeInfo.defaultPort != 0) {
+      if (port != schemeInfo.defaultPort)
+        str << ':' << port;
+    }
+
     return str;
   }
 
   // URIOnly and PathOnly
-  if (!path.IsEmpty()) {
-    PINDEX count = path.GetSize();
-    str << '/';
-    for (i = 0; i < count; i++) {
-      str << TranslateString(path[i], PathTranslation);
-      if (i < count-1)
+  if (schemeInfo.hasPath) {
+    for (i = 0; i < path.GetSize(); i++) {
+      if (i > 0 || !relativePath)
         str << '/';
+      str << TranslateString(path[i], PathTranslation);
     }
   }
+  else
+    str << TranslateString(pathStr, PathTranslation);
 
   if (fmt == URIOnly) {
     if (!fragment)
