@@ -24,6 +24,12 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: pxmlrpc.cxx,v $
+ * Revision 1.23  2003/02/21 05:07:27  robertj
+ * Fixed GetParam() for an int type so can accept i4/int/boolean type names.
+ *
+ * Revision 1.22  2003/01/28 07:42:17  robertj
+ * Improved trace output of errors.
+ *
  * Revision 1.21  2002/12/16 06:53:19  robertj
  * Added ability to specify certain elemets (by name) that are exempt from
  *   the indent formatting. Useful for XML/RPC where leading white space is
@@ -611,13 +617,15 @@ BOOL PXMLRPCBlock::GetParam(PINDEX idx, PString & result)
 BOOL PXMLRPCBlock::GetParam(PINDEX idx, int & val)
 {
   PString type, result; 
-  if (!GetExpectedParam(idx, type, result))
+  if (!GetParam(idx, type, result))
     return FALSE;
 
   if ((type != "i4") && 
       (type != "int") &&
-      (type != "boolean"))
+      (type != "boolean")) {
+    PTRACE(3, "XMLRPC\tExpected parm " << idx << " to be intger compatible, was " << type);
     return FALSE;
+  }
 
   val = result.AsInteger();
   return TRUE;
@@ -737,7 +745,11 @@ BOOL PXMLRPC::MakeRequest(const PString & method, const PXMLRPCStructBase & args
   if (!MakeRequest(request, response))
     return FALSE;
 
-  return response.GetParams(reply);
+  if (response.GetParams(reply))
+    return TRUE;
+
+  PTRACE(2, "XMLRPC\tParsing response failed: " << response.GetFaultText());
+  return FALSE;
 }
 
 
@@ -812,7 +824,7 @@ BOOL PXMLRPC::PerformRequest(PXMLRPCBlock & request, PXMLRPCBlock & response)
 
   // validate the response
   if (!response.ValidateResponse()) {
-    PTRACE(2, "XMLRPC\t" << response.GetFaultText());
+    PTRACE(2, "XMLRPC\tValidation of response failed: " << response.GetFaultText());
     return FALSE;
   }
 
