@@ -27,6 +27,14 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: pchannel.cxx,v $
+ * Revision 1.21  2003/02/11 07:22:43  robertj
+ * Fixed strange behaviour in ReadString(P_MAX_INDEX) with DOS text files where
+ *   it would get extra garbage at the end of the string, thanks Joerg Schoemer.
+ *
+ * Revision 1.20  2003/02/10 01:01:03  robertj
+ * Fixed portability issue for lseek() calls, should just look for -1 return
+ *   value to indicate error, thanks Joerg Schoemer
+ *
  * Revision 1.19  2002/12/19 03:37:05  craigs
  * Simplified PChannel::WriteString
  *
@@ -280,10 +288,16 @@ PString PChannel::ReadString(PINDEX len)
       l += lastReadCount;
     }
     str.SetSize(l+1);
-  } else {
+
+    /*Need to put in a null at the end to allow for MSDOS/Win32 text files
+      which returns fewer bytes than actually read as it shrinks the data into
+      the removed carriage returns, so it actually changes the buffer beyond
+      what it indicated. */
+    str[l] = '\0';
+  }
+  else {
     if (!ReadBlock(str.GetPointer(len+1), len))
-      return PString();
-    str.SetSize(lastReadCount+1);
+      return PString::Empty();
   }
 
   return str;
@@ -944,7 +958,7 @@ off_t PFile::GetLength() const
 #else
   off_t pos = _lseek(GetHandle(), 0, SEEK_CUR);
   off_t len = _lseek(GetHandle(), 0, SEEK_END);
-  PAssertOS(_lseek(GetHandle(), pos, SEEK_SET) == pos);
+  PAssertOS(_lseek(GetHandle(), pos, SEEK_SET) != (off_t)-1);
   return len;
 #endif
 }
@@ -962,7 +976,7 @@ BOOL PFile::SetPosition(off_t pos, FilePositionOrigin origin)
 #ifdef WOT_NO_FILESYSTEM
   return TRUE;
 #else
-  return _lseek(GetHandle(), pos, origin) == pos;
+  return _lseek(GetHandle(), pos, origin) != (off_t)-1;
 #endif
 }
 
