@@ -27,6 +27,25 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: sockets.cxx,v $
+ * Revision 1.151  2003/02/11 06:49:12  craigs
+ * Added missing OpenSocket function
+ *
+ * Revision 1.150  2003/02/03 11:23:32  robertj
+ * Added function to get pointer to IP address data.
+ *
+ * Revision 1.149  2003/02/03 10:27:55  robertj
+ * Cleaned up the gethostbyX functions for various platforms
+ *
+ * Revision 1.148  2003/02/03 08:43:01  robertj
+ * Fixed correct scoping of local variables in gethostbyX functions.
+ *
+ * Revision 1.147  2003/01/14 04:36:08  robertj
+ * Fixed v6 conversion of numeric string to binary so does not internally
+ *   doa DNS lookup, confuses other parts of the system.
+ *
+ * Revision 1.146  2003/01/11 05:10:51  robertj
+ * Fixed Win CE compatibility issues, thanks Joerg Schoemer
+ *
  * Revision 1.145  2002/12/16 08:04:46  robertj
  * Fixed correct error check for gethostbyname_r, thanks Vladimir Toncar
  *
@@ -679,7 +698,7 @@ PIPCacheData::PIPCacheData(struct hostent * host_info, const char * original)
 #ifndef _WIN32_WCE
     address = *(DWORD *)host_info->h_addr;
 #else
-    address = PIPSocket::Address(h_length, host_info->h_addr);
+    address = PIPSocket::Address(host_info->h_length, (const BYTE *)host_info->h_addr);
 #endif
   aliases.AppendString(host_info->h_name);
 
@@ -773,73 +792,71 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
   if (host == NULL) {
     mutex.Signal();
 
-#ifdef P_AIX
-    struct hostent_data ht_data;
-    memset(&ht_data, 0, sizeof(ht_data));
-    struct hostent host_info;
-#elif defined(P_RTEMS)
-    struct hostent host_info;
-#else
-    struct hostent * host_info;
-#endif
-
     int retry = 3;
     int localErrNo = NETDB_SUCCESS;
+    struct hostent * host_info;
 
+#ifdef P_AIX
+
+    struct hostent_data ht_data;
+    memset(&ht_data, 0, sizeof(ht_data));
+    struct hostent hostEnt;
     do {
-#if ( ( defined(P_PTHREADS) && !defined(P_THREAD_SAFE_CLIB) ) || (defined(__NUCLEUS_PLUS__) ) )
-      // this function should really be a static on PIPSocket, but this would
-      // require allocating thread-local storage for the data and that's too much
-      // of a pain!
+      host_info = &hostEnt;
+      ::gethostbyname_r(name,
+                        host_info,
+                        &ht_data);
+      localErrNo = h_errno;
+    } while (localErrNo == TRY_AGAIN && --retry > 0);
 
+#elif defined(P_RTEMS)
 
-#if !defined(P_AIX) && !defined(P_RTEMS)	// that I get no warnings
-      char buffer[REENTRANT_BUFFER_LEN];
-      struct hostent hostEnt;
-#endif
+    host_info = ::gethostbyname(name);
+    localErrNo = h_errno;
 
-#ifdef P_LINUX
+#elif defined P_VXWORKS
+
+    struct hostent hostEnt;
+    host_info = Vx_gethostbyname((char *)name, &hostEnt);
+    localErrNo = h_errno;
+
+#elif defined P_LINUX
+
+    char buffer[REENTRANT_BUFFER_LEN];
+    struct hostent hostEnt;
+    do {
       if (::gethostbyname_r(name,
                             &hostEnt,
                             buffer, REENTRANT_BUFFER_LEN,
                             &host_info,
       		            &localErrNo) == 0)
         localErrNo = NETDB_SUCCESS;
-      		        
-#elif defined P_AIX
-      ::gethostbyname_r(name,
-                        &host_info,
-                        &ht_data);
-      localErrNo = h_errno;
-
-#elif defined P_RTEMS
-      host_info = *::gethostbyname(name);
-#else
-      host_info = ::gethostbyname_r(name,
-			 &hostEnt, buffer, REENTRANT_BUFFER_LEN,
-			 &localErrNo);
-#endif
-
-#elif defined P_VXWORKS
-      struct hostent hostEnt;
-      host_info = Vx_gethostbyname((char *)name, &hostEnt);
-      localErrNo = h_errno;
-#else
-      host_info = ::gethostbyname(name);
-      localErrNo = h_errno;
-#endif
     } while (localErrNo == TRY_AGAIN && --retry > 0);
 
+#elif (defined(P_PTHREADS) && !defined(P_THREAD_SAFE_CLIB)) || defined(__NUCLEUS_PLUS__)
+
+    char buffer[REENTRANT_BUFFER_LEN];
+    struct hostent hostEnt;
+    do {
+      host_info = ::gethostbyname_r(name,
+                                    &hostEnt,
+                                    buffer, REENTRANT_BUFFER_LEN,
+                                    &localErrNo);
+    } while (localErrNo == TRY_AGAIN && --retry > 0);
+
+#else
+
+    host_info = ::gethostbyname(name);
+    localErrNo = h_errno;
+
+#endif
+
     mutex.Wait();
 
     if (localErrNo != NETDB_SUCCESS || retry == 0)
       return NULL;
 
-#if defined(P_AIX) || defined(P_RTEMS)
-    host = new PIPCacheData (&host_info, (const char*) name);
-#else
     host = new PIPCacheData(host_info, name);
-#endif
 
     SetAt(key, host);
   }
@@ -907,74 +924,77 @@ PIPCacheData * PHostByAddr::GetHost(const PIPSocket::Address & addr)
   if (host == NULL) {
     mutex.Signal();
 
-#ifdef P_AIX
-    struct hostent_data ht_data;
-    struct hostent host_info;
-#elif P_RTEMS
-    struct hostent host_info;
-#else
-    struct hostent * host_info;
-#endif
-
     int retry = 3;
     int localErrNo = NETDB_SUCCESS;
-    do {
-#if ( ( defined(P_PTHREADS) && !defined(P_THREAD_SAFE_CLIB) ) || ( defined(__NUCLEUS_PLUS__) ) )
-      // this function should really be a static on PIPSocket, but this would
-      // require allocating thread-local storage for the data and that's too much
-      // of a pain!
-      
-#if !defined(P_AIX) && !defined(P_RTEMS)	// that I get no warnings
-      int localErrNo;
-      char buffer[REENTRANT_BUFFER_LEN];
-      struct hostent hostEnt;
-#endif
+    struct hostent * host_info;
 
-#ifdef P_LINUX
-      ::gethostbyaddr_r((const char *)&addr, sizeof(addr),
+#ifdef P_AIX
+
+    struct hostent_data ht_data;
+    struct hostent hostEnt;
+    do {
+      host_info = &hostEnt;
+      ::gethostbyaddr_r((char *)addr.GetPointer(), addr.GetSize(),
+                        PF_INET, 
+                        host_info,
+                        &ht_data);
+      localErrNo = h_errno;
+    } while (localErrNo == TRY_AGAIN && --retry > 0);
+
+#elif defined P_RTEMS
+
+    host_info = ::gethostbyaddr(addr.GetPointer(), addr.GetSize(), PF_INET);
+    localErrNo = h_errno;
+
+#elif defined P_VXWORKS
+
+    struct hostent hostEnt;
+    host_info = Vx_gethostbyaddr(addr.GetPointer(), &hostEnt);
+
+#elif defined P_LINUX
+
+    char buffer[REENTRANT_BUFFER_LEN];
+    struct hostent hostEnt;
+    do {
+      ::gethostbyaddr_r(addr.GetPointer(), addr.GetSize(),
                         PF_INET, 
                         &hostEnt,
                         buffer, REENTRANT_BUFFER_LEN,
                         &host_info,
                         &localErrNo);
-#elif defined P_AIX
-      ::gethostbyaddr_r((char *)&addr, sizeof(addr),
-                        PF_INET, 
-                        &host_info,
-                        &ht_data );
-      localErrNo = h_errno;
-#elif defined P_RTEMS
-      host_info = *::gethostbyaddr((const char *)&addr, sizeof(addr), PF_INET);
-      localErrNo = h_errno;
+    } while (localErrNo == TRY_AGAIN && --retry > 0);
+
+#elif (defined(P_PTHREADS) && !defined(P_THREAD_SAFE_CLIB)) || defined(__NUCLEUS_PLUS__)
+
+    char buffer[REENTRANT_BUFFER_LEN];
+    struct hostent hostEnt;
+    do {
+      host_info = ::gethostbyaddr_r(addr.GetPointer(), addr.GetSize(),
+                                    PF_INET, 
+                                    &hostEnt,
+                                    buffer, REENTRANT_BUFFER_LEN,
+                                    &localErrNo);
+    } while (localErrNo == TRY_AGAIN && --retry > 0);
+
 #else
-      host_info = ::gethostbyaddr_r((const char *)&addr, sizeof(addr), PF_INET, 
-                                    &hostEnt, buffer, REENTRANT_BUFFER_LEN, &localErrNo);
+
+    host_info = ::gethostbyaddr(addr.GetPointer(), addr.GetSize(), PF_INET);
+    localErrNo = h_errno;
+
+#if defined(_WIN32) || defined(WINDOWS)  // Kludge to avoid strange 95 bug
+    extern P_IsOldWin95();
+    if (P_IsOldWin95() && host_info != NULL && host_info->h_addr_list[0] != NULL)
+      host_info->h_addr_list[1] = NULL;
 #endif
 
-#elif defined P_VXWORKS
-      struct hostent hostEnt;
-      host_info = Vx_gethostbyaddr((char *)&addr, &hostEnt);
-#else
-      host_info = ::gethostbyaddr((const char *)&addr, sizeof(addr), PF_INET);
-      localErrNo = h_errno;
-#if defined(_WIN32) || defined(WINDOWS)  // Kludge to avoid strange 95 bug
-      extern P_IsOldWin95();
-      if (P_IsOldWin95() && host_info != NULL && host_info->h_addr_list[0] != NULL)
-        host_info->h_addr_list[1] = NULL;
 #endif
-#endif
-    } while (localErrNo == TRY_AGAIN && --retry > 0);
 
     mutex.Wait();
 
     if (localErrNo != NETDB_SUCCESS || retry == 0)
       return NULL;
 
-#if defined(P_AIX) || defined(P_RTEMS)
-    host = new PIPCacheData(&host_info, addr.AsString());
-#else
     host = new PIPCacheData(host_info, addr.AsString());
-#endif
 
     SetAt(key, host);
   }
@@ -2013,13 +2033,10 @@ PIPSocket::Address & PIPSocket::Address::operator=(const PString & dotNotation)
 #if P_HAS_IPV6
 
   struct addrinfo *res;
-  struct addrinfo hints= {AI_PASSIVE, 0, SOCK_STREAM, 0, 0, NULL, NULL, NULL };
+  struct addrinfo hints = { AI_NUMERICHOST, PF_UNSPEC }; // Could be IPv4: x.x.x.x or IPv6: x:x:x:x::x
   
   version = 0;
   
-  hints.ai_family = PF_UNSPEC;//PF_INET6; Could be IPv4: x.x.x.x or IPv6: x:x:x:x::x
-  hints.ai_socktype = 0;//SOCK_STREAM;
-  
   if (getaddrinfo((const char *)dotNotation, NULL , &hints, &res) == 0) {
     if (res->ai_family == PF_INET6) {
       // IPv6 addr
@@ -2544,5 +2561,12 @@ void PUDPSocket::GetLastReceiveAddress(Address & address, WORD & port)
   port    = lastReceivePort;
 }
 
+//////////////////////////////////////////////////////////////////////////////
+
+BOOL PICMPSocket::OpenSocket(int)
+{
+  return FALSE;
+}
+
 
 // End Of File ///////////////////////////////////////////////////////////////
