@@ -27,6 +27,17 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: pipechan.cxx,v $
+ * Revision 1.41  2003/01/09 08:21:47  robertj
+ * Fixed possibly handle leak if fork() fails.
+ * Also added belt and braces checks for making sure no handle can leak in
+ *   other unknown logic. Plus do handle high water mark logging.
+ *
+ * Revision 1.40  2003/01/08 01:33:52  craigs
+ * Fixed problem with not checking errno on return from waitpid
+ *
+ * Revision 1.39  2003/01/08 01:29:22  craigs
+ * More changes for return code to waitpid
+ *
  * Revision 1.38  2002/12/18 01:12:09  craigs
  * Remove erroneous WUNTRACED and added support for EINTR from waitpid
  *
@@ -135,6 +146,9 @@
 #include "../common/pipechan.cxx"
 
 
+int PX_NewHandle(const char *, int);
+
+
 ////////////////////////////////////////////////////////////////
 //
 //  PPipeChannel
@@ -164,39 +178,52 @@ BOOL PPipeChannel::PlatformOpen(const PString & subProgram,
   // setup the pipe to the child
   if (mode == ReadOnly)
     toChildPipe[0] = toChildPipe[1] = -1;
-  else 
+  else {
     PAssert(pipe(toChildPipe) == 0, POperatingSystemError);
+    PX_NewHandle("PPipeChannel toChildPipe", PMAX(toChildPipe[0], toChildPipe[1]));
+  }
  
   // setup the pipe from the child
   if (mode == WriteOnly || mode == ReadWriteStd)
     fromChildPipe[0] = fromChildPipe[1] = -1;
-  else
+  else {
     PAssert(pipe(fromChildPipe) == 0, POperatingSystemError);
+    PX_NewHandle("PPipeChannel fromChildPipe", PMAX(fromChildPipe[0], fromChildPipe[1]));
+  }
 
   if (stderrSeparate)
     PAssert(pipe(stderrChildPipe) == 0, POperatingSystemError);
-  else
+  else {
     stderrChildPipe[0] = stderrChildPipe[1] = -1;
+    PX_NewHandle("PPipeChannel stderrChildPipe", PMAX(stderrChildPipe[0], stderrChildPipe[1]));
+  }
 
   // fork to allow us to execute the child
 #if defined(__BEOS__) || defined(P_IRIX)
-  if ((childPid = fork()) != 0) {
+  childPid = fork();
 #else
-  if ((childPid = vfork()) != 0) {
+  childPid = vfork();
 #endif
+  if (childPid < 0)
+    return FALSE;
+
+  if (childPid > 0) {
     // setup the pipe to the child
-    if (toChildPipe[0] != -1) 
+    if (toChildPipe[0] != -1) {
       ::close(toChildPipe[0]);
+      toChildPipe[0] = -1;
+    }
 
-    if (fromChildPipe[1] != -1)
+    if (fromChildPipe[1] != -1) {
       ::close(fromChildPipe[1]);
+      fromChildPipe[1] = -1;
+    }
  
-    if (stderrChildPipe[1] != -1)
+    if (stderrChildPipe[1] != -1) {
       ::close(stderrChildPipe[1]);
+      stderrChildPipe[1] = -1;
+    }
  
-    if (childPid < 0)
-      return FALSE;
-
     os_handle = 0;
     return TRUE;
   }
@@ -289,16 +316,23 @@ BOOL PPipeChannel::PlatformOpen(const PString & subProgram,
 
 BOOL PPipeChannel::Close()
 {
-  if (!IsOpen())
-    return TRUE;
-
   // close pipe from child
   if (fromChildPipe[0] != -1) {
     ::close(fromChildPipe[0]);
     fromChildPipe[0] = -1;
   }
 
+  if (fromChildPipe[1] != -1) {
+    ::close(fromChildPipe[1]);
+    fromChildPipe[1] = -1;
+  }
+
   // close pipe to child
+  if (toChildPipe[0] != -1) {
+    ::close(toChildPipe[0]);
+    toChildPipe[0] = -1;
+  }
+
   if (toChildPipe[1] != -1) {
     ::close(toChildPipe[1]);
     toChildPipe[1] = -1;
@@ -310,6 +344,11 @@ BOOL PPipeChannel::Close()
     stderrChildPipe[0] = -1;
   }
 
+  if (stderrChildPipe[1] != -1) {
+    ::close(stderrChildPipe[1]);
+    stderrChildPipe[1] = -1;
+  }
+
   // kill the child process
   if (IsRunning()) {
     kill (childPid, SIGKILL);
@@ -385,8 +424,19 @@ BOOL PPipeChannel::IsRunning() const
   PPipeChannel * thisW = (PPipeChannel *)this;
   thisW->childPid = 0;
 
-  if (WIFEXITED(status) != 0)
+  if (WIFEXITED(status)) {
     thisW->retVal = WEXITSTATUS(status);
+    PTRACE(2, "PipeChannel\tChild exited with code " << retVal);
+  } else if (WIFSIGNALED(status)) {
+    PTRACE(2, "PipeChannel\tChild was signalled with " << WTERMSIG(status));
+    thisW->retVal = -1;
+  } else if (WIFSTOPPED(status)) {
+    PTRACE(2, "PipeChannel\tChild was stopped with " << WSTOPSIG(status));
+    thisW->retVal = -1;
+  } else {
+    PTRACE(2, "PipeChannel\tChild was stopped with unknown status" << status);
+    thisW->retVal = -1;
+  }
 
   return FALSE;
 
@@ -408,13 +458,22 @@ int PPipeChannel::WaitForTermination()
     err = waitpid(childPid, &status, 0);
     if (err == childPid) {
       childPid = 0;
-      if (WIFEXITED(status) != 0)
+      if (WIFEXITED(status)) {
         retVal = WEXITSTATUS(status);
-      else
+        PTRACE(2, "PipeChannel\tChild exited with code " << retVal);
+      } else if (WIFSIGNALED(status)) {
+        PTRACE(2, "PipeChannel\tChild was signalled with " << WTERMSIG(status));
         retVal = -1;
+      } else if (WIFSTOPPED(status)) {
+        PTRACE(2, "PipeChannel\tChild was stopped with " << WSTOPSIG(status));
+        retVal = -1;
+      } else {
+        PTRACE(2, "PipeChannel\tChild was stopped with unknown status" << status);
+        retVal = -1;
+      }
       return retVal;
     }
-  } while (err == EINTR);
+  } while (errno == EINTR);
 #else
   if ((err = kill (childPid, 0)) == 0)
     return retVal = PThread::Current()->PXBlockOnChildTerminate(childPid, PMaxTimeInterval);
@@ -438,13 +497,22 @@ int PPipeChannel::WaitForTermination(const PTimeInterval & timeout)
     err = waitpid(childPid, &status, 0);
     if (err == childPid) {
       childPid = 0;
-      if (WIFEXITED(status) != 0)
+      if (WIFEXITED(status)) {
         retVal = WEXITSTATUS(status);
-      else
+        PTRACE(2, "PipeChannel\tChild exited with code " << retVal);
+      } else if (WIFSIGNALED(status)) {
+        PTRACE(2, "PipeChannel\tChild was signalled with " << WTERMSIG(status));
         retVal = -1;
+      } else if (WIFSTOPPED(status)) {
+        PTRACE(2, "PipeChannel\tChild was stopped with " << WSTOPSIG(status));
+        retVal = -1;
+      } else {
+        PTRACE(2, "PipeChannel\tChild was stopped with unknown status" << status);
+        retVal = -1;
+      }
       return retVal;
     }
-  } while (err == EINTR);
+  } while (errno == EINTR);
 #else
   if ((err = kill (childPid, 0)) == 0)
     return retVal = PThread::Current()->PXBlockOnChildTerminate(childPid, timeout);
