@@ -27,6 +27,33 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: tlibthrd.cxx,v $
+ * Revision 1.115  2003/03/07 00:07:15  robertj
+ * Fixed Mac OS X patch which broke every other platform.
+ *
+ * Revision 1.114  2003/03/06 08:58:48  rogerh
+ * P_MACOSX now carries the OSRELEASE. Use this to enable better threads
+ * support on Darwin 6.4. Submitted by Shawn.
+ *
+ * Revision 1.113  2003/02/20 23:32:00  robertj
+ * More RTEMS support patches, thanks Sebastian Meyer.
+ *
+ * Revision 1.112  2003/01/24 10:21:06  robertj
+ * Fixed issues in RTEMS support, thanks Vladimir Nesic
+ *
+ * Revision 1.111  2003/01/20 10:13:18  rogerh
+ * NetBSD thread changes
+ *
+ * Revision 1.110  2003/01/20 10:05:46  rogerh
+ * NetBSD thread changes
+ *
+ * Revision 1.109  2003/01/08 08:47:51  rogerh
+ * Add new Sleep() function for GNU PTH threads.
+ * Taken from NetBSD's package which uses PTH.
+ * Note: I am not sure this works correctly.
+ *
+ * Revision 1.108  2003/01/06 18:49:15  rogerh
+ * Back out pthead_kill to pthread_cancel change on NetBSD
+ *
  * Revision 1.107  2002/12/11 05:39:26  robertj
  * Added logging for file handle changes.
  * Fixd bug where internal maxHandles not set when increased.
@@ -420,7 +447,7 @@ static BOOL PAssertThreadOp(int retval,
   if (errno == EINTR || errno == EAGAIN) {
     if (++retry < 1000) {
 #if defined(P_RTEMS)
-      rtems_task_wake_after(10);
+      sched_yield();
 #else
       usleep(10000); // Basically just swap out thread to try and clear blockage
 #endif
@@ -504,6 +531,7 @@ void PProcess::Construct()
 
   ::pipe(timerChangePipe);
 #else
+  maxHandles = 500; // arbitrary value
   socketpair(AF_INET,SOCK_STREAM,0,timerChangePipe);
 #endif
 
@@ -654,7 +682,7 @@ void PThread::Restart()
   pthread_attr_init(&threadAttr);
   pthread_attr_setdetachstate(&threadAttr, PTHREAD_CREATE_DETACHED);
 
-#if defined(P_LINUX) || defined(P_RTEMS)
+#if defined(P_LINUX)
   /*
     Set realtime scheduling if our effective user id is root (only then is this
     allowed) AND our priority is Highest.
@@ -668,6 +696,13 @@ void PThread::Restart()
   */
   if ((geteuid() == 0) && (PX_priority == HighestPriority))
     PAssertPTHREAD(pthread_attr_setschedpolicy, (&threadAttr, SCHED_FIFO));
+#elif defined(P_RTEMS)
+  pthread_attr_setstacksize(&threadAttr, 2*PTHREAD_MINIMUM_STACK_SIZE);
+  pthread_attr_setinheritsched(&threadAttr, PTHREAD_EXPLICIT_SCHED);
+  pthread_attr_setschedpolicy(&threadAttr, SCHED_OTHER);
+  struct sched_param sched_param;
+  sched_param.sched_priority = 125; /* set medium priority */
+  pthread_attr_setschedparam(&threadAttr, &sched_param);
 #endif
 
   PAssertPTHREAD(pthread_create, (&PX_threadId, &threadAttr, PX_ThreadStart, this));
@@ -684,9 +719,9 @@ void PX_SuspendSignalHandler(int)
   while (notResumed) {
     BYTE ch;
     notResumed = ::read(thread->unblockPipe[0], &ch, 1) < 0 && errno == EINTR;
-#ifndef P_NETBSD
+#if !( defined(P_NETBSD) && defined(P_NO_CANCEL) )
     pthread_testcancel();
-#endif /* P_NETBSD */
+#endif
   }
 }
 
@@ -712,7 +747,7 @@ void PThread::Suspend(BOOL susp)
     return;
   }
 
-#ifdef P_MACOSX
+#if defined(P_MAXOSX) && (P_MACOSX <= 55)
   // Suspend - warn the user with an Assertion
   PAssertAlways("Cannot suspend threads on Mac OS X due to lack of pthread_kill()");
 #else
@@ -776,7 +811,7 @@ void PThread::SetPriority(Priority priorityLevel)
 {
   PX_priority = priorityLevel;
 
-#if defined(P_LINUX) || defined(P_RTEMS)
+#if defined(P_LINUX)
   if (IsTerminated())
     return;
 
@@ -799,7 +834,7 @@ void PThread::SetPriority(Priority priorityLevel)
 
 PThread::Priority PThread::GetPriority() const
 {
-#if defined(LINUX) || defined(P_RTEMS)
+#if defined(LINUX)
   int schedulingPolicy;
   struct sched_param schedParams;
   
@@ -834,6 +869,30 @@ void PThread::PXSetWaitingSemaphore(PSemaphore * sem)
 #endif
 
 
+#ifdef P_GNU_PTH
+// GNU PTH threads version (used by NetBSD)
+// Taken from NetBSD pkg patches
+void PThread::Sleep(const PTimeInterval & timeout)
+{
+  PTime lastTime;
+  PTime targetTime = PTime() + timeout;
+
+  sched_yield();
+  lastTime = PTime();
+
+  while (lastTime < targetTime) {
+    P_timeval tval = targetTime - lastTime;
+    if (select(0, NULL, NULL, NULL, tval) < 0 && errno != EINTR)
+      break;
+
+    pthread_testcancel();
+
+    lastTime = PTime();
+  }
+}
+
+#else
+// Normal Posix threads version
 void PThread::Sleep(const PTimeInterval & timeout)
 {
   PTime lastTime;
@@ -843,14 +902,14 @@ void PThread::Sleep(const PTimeInterval & timeout)
     if (select(0, NULL, NULL, NULL, tval) < 0 && errno != EINTR)
       break;
 
-#ifndef P_NETBSD
+#if !( defined(P_NETBSD) && defined(P_NO_CANCEL) )
     pthread_testcancel();
-#endif /* P_NETBSD */
+#endif
 
     lastTime = PTime();
   } while (lastTime < targetTime);
 }
-
+#endif
 
 void PThread::Yield()
 {
@@ -899,7 +958,11 @@ void PThread::Terminate()
   PAssertPTHREAD(pthread_mutex_unlock, (&PX_WaitSemMutex));
 #endif
 
+#if ( defined(P_NETBSD) && defined(P_NO_CANCEL) )
+  pthread_kill(PX_threadId,SIGKILL);
+#else
   pthread_cancel(PX_threadId);
+#endif
 }
 
 
@@ -908,7 +971,7 @@ BOOL PThread::IsTerminated() const
   if (PX_threadId == 0)
     return TRUE;
 
-#ifndef P_MACOSX
+#if defined(P_MAXOSX) && (P_MACOSX <= 55)
   // MacOS X does not support pthread_kill so we cannot use it
   // to test the validity of the thread
   if (pthread_kill(PX_threadId, 0) != 0)
